﻿using System;
using System.Windows.Forms;
using System.Globalization; //for NumberStyles format specifiers

// you must also add a reference to the AIOUSBNet.dll in the project
// The path should be C:\Windows\system32\AIOUSBNet.dll or the equivalent for your system


namespace Sample1
{
    public partial class FormSample1 : Form
    {
        // Array of controls for GUI updates:
        public CheckBox[] checkOutArray = new CheckBox[16];
        public CheckBox[] checkInArray = new CheckBox[16];
        public TextBox[] txtWriteArray = new TextBox[2];

        public UInt32 DeviceIndex;

        // Interval Timer for GUI update:
        // (Note: A Windows Forms timer designed for single threaded environments not a System Timer)
        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();
        private double GlobalTickHz;

        public FormSample1()
        {
            InitializeComponent();
            
            // Additional Constructor Initializations here:

            txtWriteArray[0] = txtWriteByte0;
            txtWriteArray[1] = txtWriteByte1;

            // Array of the LED style checkboxes:
            checkOutArray[0] = checkOutA0;
            checkOutArray[1] = checkOutA1;
            checkOutArray[2] = checkOutA2;
            checkOutArray[3] = checkOutA3;
            checkOutArray[4] = checkOutA4;
            checkOutArray[5] = checkOutA5;
            checkOutArray[6] = checkOutA6;
            checkOutArray[7] = checkOutA7;
            checkOutArray[8] = checkOutB8;
            checkOutArray[9] = checkOutB9;
            checkOutArray[10] = checkOutB10;
            checkOutArray[11] = checkOutB11;
            checkOutArray[12] = checkOutB12;
            checkOutArray[13] = checkOutB13;
            checkOutArray[14] = checkOutB14;
            checkOutArray[15] = checkOutB15;
            
            checkInArray[0] = checkInA0;
            checkInArray[1] = checkInA1;
            checkInArray[2] = checkInA2;
            checkInArray[3] = checkInA3;
            checkInArray[4] = checkInA4;
            checkInArray[5] = checkInA5;
            checkInArray[6] = checkInA6;
            checkInArray[7] = checkInA7;
            checkInArray[8] = checkInB8;
            checkInArray[9] = checkInB9;
            checkInArray[10] = checkInB10;
            checkInArray[11] = checkInB11;
            checkInArray[12] = checkInB12;
            checkInArray[13] = checkInB13;
            checkInArray[14] = checkInB14;
            checkInArray[15] = checkInB15;
        }
        //**************************************************************************************************
        // A few helper functions

        private bool HandleTryParseResult(bool bResult)
        {
            if (!bResult)
            {
                txtStatus.Text = "Format: Enter a hex number 00-FF valid hex values 0-9, A-F and a-f, no 0x prefix!";
                return false;
            }
            else
                txtStatus.Text = "";

            return true;
        }

        private void WriteHexStringToDIOAll(string HexString)
        {
            UInt32 bits = 0; // note, this changes based on how many bytes of DIO the card supports
            if (HandleTryParseResult(UInt32.TryParse(HexString, NumberStyles.HexNumber, null as IFormatProvider, out bits)))
                AIOUSB.DIO_WriteAll(DeviceIndex, ref bits);
        }

        private void WriteHexStringToDIOByte(string HexString, byte ByteIndex)
        {
            Byte bits = 0;
            if (HandleTryParseResult(Byte.TryParse(HexString, NumberStyles.HexNumber, null as IFormatProvider, out bits)))
                AIOUSB.DIO_Write8(DeviceIndex, ByteIndex, bits);
        }
        // end of helper functions
        //**************************************************************************************************

        private void FormSample1_Load(object sender, EventArgs e)
        {
            DeviceIndex = AIOUSB.diFirst;   // Default to first device found
            myTimer.Tick += new EventHandler(TimerEventProcessor);

            myTimer.Interval = 100;
            myTimer.Start();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            // Right now this is only called on button click not window close ALtF4 etc:
            
            // Clean up and Close App:
            UInt32 Status;

            Status = AIOUSB.ClearDevices();

            // Kill the timer:
            myTimer.Stop();
            myTimer.Enabled = false;
            myTimer.Dispose();

            // Close the form and Application:
            this.Close();
        }
  
        private void btnDevice_Click(object sender, EventArgs e)
        {
        }

        private void btnWritePortAll_Click(object sender, EventArgs e)
        {
            WriteHexStringToDIOAll(txtWriteAll.Text);
        }

          private void btnOutRelay_Click(object sender, EventArgs e)
        {
            // Button Clicks to toggle the relay and hence display states:
            byte stateByte = 1;
            UInt32 bits = 0; // Note, this would change for device-firmwares with different numbers of DIO bytes

            // Get sender's Tag (bitNumber)
            byte bitNumber = Convert.ToByte((sender as Button).Tag);

            // Read All then test correct bit and set individual relay state:
            AIOUSB.DIO_ReadAll(DeviceIndex, bits);

            if ((bits & (1 << bitNumber)) != 0)
                stateByte = 0;
            else
                stateByte = 1;

            AIOUSB.DIO_Write1(DeviceIndex, bitNumber, stateByte);
            
            // NOTE:
            // You can also use DIO_WriteAll(), which is much faster than even just two writes to DIO_Write1:
            //      result = AIOUSB.DIO_ReadAll(DeviceIndex, out bits);
            //      result = AIOUSB.DIO_WriteAll(DeviceIndex, bits ^ (1 << bitNumber)); // toggle using XOR
            // A single call to DIO_Write1() is almost exactly the same speed (actually a few cpu cycles SLOWER)
            // than a single call to DIO_WriteAll() -- but DIO_Write1 is often EASIER
        }

        private void btnWritePortByte_Click(object sender, EventArgs e)
        {
            Byte portIndex = Convert.ToByte( (sender as Button).Tag);
            WriteHexStringToDIOByte(txtWriteArray[portIndex].Text,  portIndex); 
        }

        private void TimerEventProcessor(Object myObject, EventArgs myEventArgs)
        {
            // This is the method that runs when the timer event is raised:           
            UInt32 result;
            byte[] bits=new byte[4];

            myTimer.Stop();

            result = AIOUSB.DIO_ReadAll(DeviceIndex, bits);

            // Display Hex format:
            txtReadByte3.Text = bits[3].ToString("X2");
            txtReadByte2.Text = bits[2].ToString("X2");
            txtReadAllHex.Text = bits[0].ToString("X2") + bits[1].ToString("X2") + txtReadByte2.Text + txtReadByte3.Text ;

            // Display Binary format:
            txtReadAllBin.Text = Convert.ToString(bits[3], 2).PadLeft(8, '0') + Convert.ToString(bits[2], 2).PadLeft(8, '0');
            
            for (int i = 0; i < 8; i++)
            {
                // display current output values (DIO_ReadAll bits 0-15)
                if( (bits[0] & ( 1 << (i) )) != 0 ) 
                    checkOutArray[i].Checked = true; 
                else
                    checkOutArray[i].Checked = false;

                if( (bits[1] & ( 1 << (i) )) != 0 ) 
                    checkOutArray[i+8].Checked = true; 
                else
                    checkOutArray[i+8].Checked = false;

                // display current input values (DIO_ReadAll bits 16->31)
                if ((bits[2] & (1 << (i))) != 0) 
                    checkInArray[i].Checked = true; 
                else
                    checkInArray[i].Checked = false; 

                if ((bits[3] & (1 << (i))) != 0) 
                    checkInArray[i+8].Checked = true; 
                else
                    checkInArray[i+8].Checked = false; 
            }
            myTimer.Enabled = true;
        }

        private void nupdnGlobalTickDivisor_ValueChanged(object sender, EventArgs e)
        {
            try {
                ushort Divisor = Convert.ToUInt16(nupdnDivisor.Text.ToString());
                GlobalTickHz = 12000000.0 / Divisor;
                labGlobalTickHz.Text = (Math.Round(GlobalTickHz * 100) / 100).ToString() + " Hz Tick Rate";
                labInterval.Text = (Math.Round(100000 / GlobalTickHz) / 100).ToString() + " ms between global ticks";
            }
            catch { }
            try
            {
                byte Divisor = Convert.ToByte(nupdnLatchDivisor.Text.ToString());
                double Hz = GlobalTickHz / Divisor;
                labLatchHz.Text = (Math.Round(Hz * 100) / 100).ToString() + " Hz Latch Rate";
                labLatchInterval.Text = (Math.Round(10000 / Hz) / 10).ToString() + " ms between latch events";
            }
            catch { }
        }

        private void butGlobalTickConfigure_Click(object sender, EventArgs e)
        {
            ushort Divisor = Convert.ToUInt16(nupdnDivisor.Text.ToString());
            UInt32 Status = AIOUSB.GenericVendorWrite(DeviceIndex, 0x3c, Divisor, 0x8001, 0, null);
            if (Status == 0)
                txtStatus.Text = "successfully wrote global tick";
            else txtStatus.Text = "problem writing global tick " + Status.ToString();
        }

        private void butLatchDivisorConfigure_Click(object sender, EventArgs e)
        {
            byte Divisor = Convert.ToByte(nupdnLatchDivisor.Text.ToString());
            byte[] buf = new byte[1];
            buf[0] = Divisor;
            UInt32 len = 1;
            UInt32 Status = AIOUSB.GenericVendorWrite(DeviceIndex, 0x3c, 0, 0x0063, len, buf);
            if (Status == 0)
                txtStatus.Text = "successfully wrote Latch divisor";
            else txtStatus.Text = "problem writing Latch Divisor " + Status.ToString();
        }
        private void butReadLatches_Click(object sender, EventArgs e)
        {
            byte[] buf = new byte[12];
            UInt32 len = 12;

            UInt32 Status = AIOUSB.GenericVendorRead(DeviceIndex, 0x14, 0, 0, ref len, buf);
            if (Status != 0) labLowLatches.Text = "error " + Status.ToString();
            else
            {
                labLowLatches.Text = buf[0].ToString("X2") + buf[1].ToString("X2") + " " + buf[2].ToString("X2") + buf[3].ToString("X2");
                labHighLatches.Text = buf[4].ToString("X2") + buf[5].ToString("X2") + " " +buf[6].ToString("X2") + buf[7].ToString("X2");
                labLatchedData.Text = buf[8].ToString("X2") + buf[9].ToString("X2") + " "+buf[10].ToString("X2") + buf[11].ToString("X2");

            }


        }

    }
}
